# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
"""
Version information.
"""

__version__ = '0.4'
__version_info__ = tuple(int(x) for x in __version__.split('.'))
